package zero1hd.wavedecoder;

import java.io.IOException;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

import com.badlogic.gdx.files.FileHandle;

public class WavDecoder {
	private FileHandle file;
	
	private int channels;
	private double sampleRate;
	private int dataSize;
	private int byteRate;
	private String fileName;
	private byte[] buffer;
	private AudioInputStream audioInputStream;
	public WavDecoder(FileHandle file) throws IOException {
		this.file = file;
		try {
			audioInputStream = AudioSystem.getAudioInputStream(file.read());
			buffer = new byte[audioInputStream.getFormat().getFrameSize()];
		} catch (UnsupportedAudioFileException e) {
			e.printStackTrace();
		}
	}
	
	public int getChannels() {
		return channels;
	}
	
	public int getByteRate() {
		return byteRate;
	}
	
	public int getDataSize() {
		return dataSize;
	}
	
	public double getSampleRate() {
		return sampleRate;
	}
	
	public long getDurationInSeconds() {
		return (long) (dataSize/byteRate);
	}
	
	public String getFileName() {
		return fileName;
	}
	
	public FileHandle getFile() {
		return file;
	}
	
	public int readSamples(float[] samples) throws IOException {
		int samplesRead = 0;
		
		for (int sampleID = 0; sampleID < samples.length; sampleID++) {
			audioInputStream.read(buffer);
			
			samples[sampleID] += (short) ((buffer[1] << 8) + (buffer[0] & 0xff));
			if (audioInputStream.getFormat().getChannels() > 1) {
				samples[sampleID] = (short) ((buffer[3] << 8) + (buffer[2] & 0xff));
				samples[sampleID] /= 2;
			}
			
			samples[sampleID] /= Short.MAX_VALUE+1;
			samplesRead ++;
		}
		
		return samplesRead;
	}
	
	public void cleanAndClose() {
		try {
			audioInputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
